/*
    XorDev's "Default Shaderpack"

    This was put together by @XorDev to make it easier for anyone to make their own shaderpacks in Minecraft (Optifine).
    You can do whatever you want with this code! Credit is not necessary, but always appreciated!

    You can find more information about shaders in Optfine here:
    https://github.com/sp614x/optifine/blob/master/OptiFineDoc/doc/shaders.txt

*/
//Declare GL version.
#version 150 compatibility

//Diffuse (color) texture.
uniform sampler2D texture;
//Lighting from day/night + shadows + light sources.
uniform sampler2D lightmap;

uniform sampler2DShadow shadowtex0;
uniform sampler2DShadow shadowtex1;
uniform sampler2D shadowcolor;

uniform mat4 shadowProjection;
uniform mat4 shadowModelView;

uniform float viewWidth;
uniform float viewHeight;
uniform sampler2D noisetex;

//RGB/intensity for hurt entities and flashing creepers.
uniform vec4 entityColor;
//0-1 amount of blindness.
uniform float blindness;
//0 = default, 1 = water, 2 = lava.
uniform int isEyeInWater;

//Vertex color.
varying vec4 color;
//Diffuse and lightmap texture coordinates.
varying vec2 coord0;
varying vec2 coord1;

varying vec4 shadowPos;

#include "/common.glsl"

const float PI = radians(180.0);

vec2 vogelDiscSample(int stepIndex, int stepCount, float noise) {
    float rotation = noise * 2 * PI;
    const float goldenAngle = 2.4;

    float r = sqrt(stepIndex + 0.5) / sqrt(float(stepCount));
    float theta = stepIndex * goldenAngle + rotation;

    return r * vec2(cos(theta), sin(theta));
}

float interleavedGradientNoise(vec2 coord){
	return fract(52.9829189 * fract(0.06711056 * coord.x + (0.00583715 * coord.y)));
}

void main()
{
    vec2 coord1 = coord1;
    if(shadowPos != vec4(0.0)){
        float shadowAccum = 0.0;
        const float sampleRadius = SHADOW_SOFTNESS / shadowMapResolution;
        float noise = interleavedGradientNoise(floor(gl_FragCoord.xy));
        for(int i = 0; i < SHADOW_SAMPLES; i++){
            vec2 offset = vogelDiscSample(i, SHADOW_SAMPLES, noise) * sampleRadius;
            #if TRANSPARENT_SHADOWS == 2
            vec3 offsetPos = shadowPos.xyz + vec3(offset, 0.0);
            shadowAccum += mix(shadow2D(shadowtex1, shadowPos.xyz + vec3(offset, 0.0)).r * (1.0 - texture2D(shadowcolor, offsetPos.xy).a), 1.0, shadow2D(shadowtex0, shadowPos.xyz + vec3(offset, 0.0)).r);
            #elif TRANSPARENT_SHADOWS == 1
            shadowAccum += shadow2D(shadowtex1, shadowPos.xyz + vec3(offset, 0.0)).r;
            #else
            shadowAccum += shadow2D(shadowtex0, shadowPos.xyz + vec3(offset, 0.0)).r;
            #endif
        }
        shadowAccum /= float(SHADOW_SAMPLES);

        coord1.y *= mix(1.0 - SHADOW_STRENGTH, 1.0, shadowAccum);
    }

    //Combine lightmap with blindness.
    vec3 light = (1.-blindness) * texture2D(lightmap,coord1).rgb;
    //Sample texture times lighting.
    vec4 col = color * vec4(light,1) * texture2D(texture,coord0);
    //Apply entity flashes.
    col.rgb = mix(col.rgb,entityColor.rgb,entityColor.a);

    //Calculate fog intensity in or out of water.
    float fog = (isEyeInWater>0) ? 1.-exp(-gl_FogFragCoord * gl_Fog.density):
    clamp((gl_FogFragCoord-gl_Fog.start) * gl_Fog.scale, 0., 1.);

    //Apply the fog.
    col.rgb = mix(col.rgb, gl_Fog.color.rgb, fog);

    //Output the result.
    gl_FragData[0] = col;
}
